require "spec_helper"

describe "Helper" do
  did = "33190074"
  binding_id = "ABCDEFGHIJKLMN"

  before(:all) do
    response = PolylinkApi::Helper.create_v2c_did(binding_id, "18101801755", "18616015606", did)
  end

  after(:all) do
    response = PolylinkApi::Helper.delete_v2c_did(binding_id)
  end

  it "should bind get client" do
    expect(PolylinkApi::Helper.client).not_to be_nil
    expect(PolylinkApi::Helper.client).to be_an_instance_of(Savon::Client)
  end

  it "should call a remote method" do
    expect(PolylinkApi::Helper.respond_to?(:call)).to be true
    expect { PolylinkApi::Helper.call(:unknown) }.to raise_error(PolylinkApi::InvalidMethodError)
  end

  it "should get response code" do
    [ "500", "501", "502", "510", "511", "517", "518", "519", "600" ].each do |code|
      expect(PolylinkApi::Helper.get_response_code(code)).to eq(code)
      expect(PolylinkApi::Helper.get_response_code("<Result>#{code}</Result><CDR></CDR>")).to eq(code)
    end
  end

  it "should check response include data" do
    [ "500", "501", "502", "510", "511", "516", "517", "518", "519", "600" ].each do |code|
      expect(PolylinkApi::Helper.response_include_data?(code)).to be false
      expect(PolylinkApi::Helper.response_include_data?("<Result>#{code}</Result><CDR></CDR>")).to be true
    end
    expect(PolylinkApi::Helper.response_include_data?("1344")).to be false
  end

  it "should check response" do
    expect { PolylinkApi::Helper.get_response(:unknown, nil) }.to raise_error(PolylinkApi::InvalidResponseError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "501" } }) }.to raise_error(PolylinkApi::AuthenticationError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "502" } }) }.to raise_error(PolylinkApi::InvalidParameterError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "510" } }) }.to raise_error(PolylinkApi::InvalidParameterError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "511" } }) }.to raise_error(PolylinkApi::InvalidParameterError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "516" } }) }.to raise_error(PolylinkApi::AlreadyExistedError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "517" } }) }.to raise_error(PolylinkApi::NotExistError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "518" } }) }.to raise_error(PolylinkApi::NotExistError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "519" } }) }.to raise_error(PolylinkApi::NotExistError)
    expect { PolylinkApi::Helper.get_response(:unknown, { unknown_response: { out: "600" } }) }.to raise_error(PolylinkApi::InvalidResponseError)
    data = { unknown_response: { out: "500" } }
    expect(PolylinkApi::Helper.get_response(:unknown, data)).to eq(nil)

    PolylinkApi::Helper::METHODS_RESPONSE_KEY[:unknown] = "TEST"
    data = { unknown_response: { out: "<Result>500</Result><TEST>blah</TEST>" } }
    expect(PolylinkApi::Helper.get_response(:unknown, data)).to eq("blah")
  end

  it "should get response data" do
    data = "test"
    expect(PolylinkApi::Helper.get_response_data(:unknown, data)).to be_nil
    PolylinkApi::Helper::METHODS_RESPONSE_KEY[:unknown] = "TEST"
    data = "<Result>500</Result><TEST>blah</TEST>"
    expect(PolylinkApi::Helper.get_response_data(:unknown, data)).to eq("blah")
  end

  it "should find bind_id" do
    response = PolylinkApi::Helper.get_v2c_id("18101801755", "18616015606", did)
    expect(response).to eq(binding_id)
  end

  it "should dial via v2c id" do
    response = PolylinkApi::Helper.callback_v2c_id(binding_id)
    expect(response).to be_nil
  end

  it "should bind default phone to did, and unbind after check pass" do
    response = PolylinkApi::Helper.bind_did_default_phone(did, "18101673107")
    expect(response).to be_nil
    response = PolylinkApi::Helper.get_did_default_phones_list
    expect(response).to be_an_instance_of(Array)
    expect(response.length).to be >= 1
    expect(response[0].keys).to eq([:did, :default_phone]) 
    response = PolylinkApi::Helper.unbind_did_default_phone(did)
    expect(response).to be_nil
  end

  it "should get cdr from string" do
    cdr = PolylinkApi::Helper.get_cdr_from_string("did,v2c_id,device_name,calling_number,called_number,starttime,duration,endtime,recordpath,call_status")
    expect(cdr).to be_an_instance_of(Hash)
    expect(cdr[:did]).to eq("did")
    expect(cdr[:v2c_id]).to eq("v2c_id")
    expect(cdr[:device_name]).to eq("device_name")
    expect(cdr[:calling_number]).to eq("calling_number")
    expect(cdr[:called_number]).to eq("called_number")
    expect(cdr[:starttime]).to eq("starttime")
    expect(cdr[:duration]).to eq("duration")
    expect(cdr[:endtime]).to eq("endtime")
    expect(cdr[:recordpath]).to eq("recordpath")
    expect(cdr[:call_status]).to eq("call_status")
    expect(cdr.keys).to eq([:did, :v2c_id, :device_name, :calling_number, :called_number, :starttime, :duration, :endtime, :recordpath, :call_status])
  end

  it "should get cdrs from string" do
    cdrs = PolylinkApi::Helper.get_cdrs_from_string("did,v2c_id,device_name,calling_number,called_number,starttime,duration,endtime,recordpath,call_status;did,v2c_id,device_name,calling_number,called_number,starttime,duration,endtime,recordpath,call_status;")
    expect(cdrs).to be_an_instance_of(Array)
    expect(cdrs.length).to eq(2)
  end

  it "should get cdr records" do
    response = PolylinkApi::Helper.get_cdrs(did, 1, Time.now - 2.hours, Time.now + 2.hours)
    expect(response).to be_an_instance_of(Array)
    expect(response.length).to be >= 1
  end

  it "should get cdr file" do
    file_cdr = "20150202/20150202162456_18101801755_18616015606_33190074_58.WAV"
    file = PolylinkApi::Helper.get_cdr_file(file_cdr)
    expect(file).to eq(File.join(PolylinkApi.config.cdrs_path, file_cdr))
    expect(File.exists?(File.join(PolylinkApi.config.cdrs_path, file_cdr))).to be true
  end

  it "should upload did voice" do
    file = File.join("spec", "config", "test_voice.wav")
    response = PolylinkApi::Helper.set_did_announcement(did, file)
    expect(response).to be_nil
  end

  it "should remove did voice" do
    response = PolylinkApi::Helper.remove_did_announcement(did)
    expect(response).to be_nil
  end

  it "should get default did" do
    response = PolylinkApi::Helper.bind_did_default_did(did, "33190084")
    expect(response).to be_nil
    response = PolylinkApi::Helper.get_did_default_dids_list
    expect(response).to be_an_instance_of(Array)
    expect(response.length).to be >= 1
    expect(response[0].keys).to eq([:did, :default_did])
    response = PolylinkApi::Helper.unbind_did_default_did(did)
    expect(response).to be_nil
  end

  it "should get did default dids" do
    response = PolylinkApi::Helper.get_dids
    expect(response).to be_an_instance_of(Array)
    expect(response.length).to be >= 1
  end
end